/*
 *  File....... IRReceiverCircuit.pde 
 *  Purpose.... Records up to 192 signal changes and prints in format for use with TransmitIRSignal 
 *  Author..... Walter Anderson, modified by Tom Lauwers
 *  E-mail..... wandrson@walteranderson.us/tlauwers@birdbraintechnologies.com
 *  Started.... 18 May 2007 
 *  Updated.... 21 Dec 2011
 *  Original at: http://www.arduino.cc/playground/Code/InfraredReceivers
 * 
 *
 */ 
#include 
#include 

#define TIMER_RESET  TCNT1 = 0
#define SAMPLE_SIZE  96 

// Sets the pin the receiver is on
int IRpin = 2;
unsigned int TimerValue[SAMPLE_SIZE]; // Array for time data
char direction[SAMPLE_SIZE]; // Array for direction data
byte change_count;
long time;
int timeout = 0; // Times out if no signal detected for 15ms or so
int signal_length = 0;
boolean timedOut = false;

void setup() {
  Serial.begin(115200);
  Serial.println("Analyze IR Remote");
  TCCR1A = 0x00;          // COM1A1=0, COM1A0=0 => Disconnect Pin OC1 from Timer/Counter 1 -- PWM11=0,PWM10=0 => PWM Operation disabled
  // ICNC1=0 => Capture Noise Canceler disabled -- ICES1=0 => Input Capture Edge Select (not used) -- CTC1=0 => Clear Timer/Counter 1 on Compare/Match
  // CS12=0 CS11=1 CS10=1 => Set prescaler to clock/64
  TCCR1B = 0x03;          // 16MHz clock with prescaler means TCNT1 increments every 4uS
  // ICIE1=0 => Timer/Counter 1, Input Capture Interrupt Enable -- OCIE1A=0 => Output Compare A Match Interrupt Enable -- OCIE1B=0 => Output Compare B Match Interrupt Enable
  // TOIE1=0 => Timer 1 Overflow Interrupt Enable
  TIMSK1 = 0x00;          
  pinMode(IRpin, INPUT);
}

void loop()
{
  Serial.println("Waiting...");
  // Reset counters and flags
  change_count = 0;
  timeout = 0;
  timedOut = false;
  // Busy loop - just hangs until our IR receiver goes low (indicating signal detected)
  while(digitalRead(IRpin) == HIGH) {}                                 
  TIMER_RESET; // Resets the TCNT1 counter to 0
  TimerValue[change_count] = TCNT1;
  direction[change_count++] = '0';
  while (change_count < SAMPLE_SIZE) {
    // If the signal was most recently 0, wait as long as the pin is low
    if (direction[change_count-1] == '0') {
      while(digitalRead(IRpin) == LOW) {}
      // Once the pin goes high (because we're out of the busy loop), record timer value and signal direction
      TimerValue[change_count] = TCNT1;
      direction[change_count++] = '1';
    } 
    // If the signal was most recently a 1, wait if the pin is low and check for a timeout
    else {
      timeout = 0;
      // We only need to check for timeout if the pin is high because receivers are high
      // when not receiving signal
      while(digitalRead(IRpin) == HIGH) {
        timeout++;
        delayMicroseconds(4);
        if(timeout > 2000) {
          signal_length = change_count;
          timedOut = true;
          break;
        }
      }
      // If we timed out, get out of this loop
      if(timedOut) {
        break;
      }
      else {
        TimerValue[change_count] = TCNT1;
        direction[change_count++] = '0';
      }
    }
  }
  Serial.println("Bit stream detected!");
  Serial.println("For use with TransmitIRSignal:");
  change_count = 1;
  // Print the data as used by TransmitIRSignal
  while (change_count < signal_length) {
    time = (long) (TimerValue[change_count]-TimerValue[change_count-1]) * 4;
    Serial.print(time);
    Serial.print(",");
    change_count++;
  }
  Serial.println("\nFor easy plotting purposes:");  
  change_count = 0;
  time = (long) TimerValue[change_count] * 4;
  Serial.print(time);
  Serial.print("\t");
  Serial.println(direction[change_count++]);
  while (change_count < signal_length) {
    time = (long) TimerValue[change_count] * 4;
    Serial.print(time);
    Serial.print("\t");
    Serial.println(direction[change_count-1]);
    Serial.print(time);
    Serial.print("\t");
    Serial.println(direction[change_count++]);    
  }
  Serial.println("Bit stream end!");
  delay(2000);
}
